from src.core.exceptions import (
    CommandNotFoundException,
    ActionNotFoundException,
)
from src.core.git import get_config

BLACK = "\033[0;30m"
RED = "\033[0;31m"
GREEN = "\033[0;32m"
BROWN_ORANGE = "\033[0;33m"
BLUE = "\033[0;34m"
PURPLE = "\033[0;35m"
CYAN = "\033[0;36m"
LIGHT_GRAY = "\033[0;37m"
DARK_GRAY = "\033[1;30m"
LIGHT_RED = "\033[1;31m"
LIGHT_GREEN = "\033[1;32m"
YELLOW = "\033[1;33m"
LIGHT_BLUE = "\033[1;34m"
LIGHT_PURPLE = "\033[1;35m"
LIGHT_CYAN = "\033[1;36m"
WHITE = "\033[1;37m"
NC = "\033[0m"  # No Color


def get_template():
    return get_config("phoenix.template-path")


def get_command(commands, command_name):
    print(f"Validating command {command_name}...")

    command = next(
        filter(lambda x: x.get("name") == command_name, commands), None
    )

    if not command:
        raise CommandNotFoundException(command_name)

    return command


def get_action(actions, action_name):
    print(f"Validating action {action_name}...")

    action = next(
        filter(lambda x: x.get("name") == action_name, actions), None
    )

    if not action:
        raise ActionNotFoundException(action_name)

    return action


def read_input(cls, msg):

    user_input = None

    while not user_input:
        try:
            print(msg)
            user_input = input()
        except EOFError:
            print(f"Please, {msg}")
        except KeyboardInterrupt:
            print("Stopping execution")

    return user_input


def define_pattern(pattern, should_start, should_end):
    if isinstance(pattern, list):
        for i in range(len(pattern)):
            pattern[i] = (
                pattern[i].replace("^", "").replace("$", "").replace("/", "\/")
            )

        pattern = "".join(pattern)

    if should_start:
        pattern = "^" + pattern
    if should_end:
        pattern = pattern + "$"

    return pattern


def determine_pattern(pattern):
    if isinstance(pattern, list):
        for i in range(len(pattern)):
            pattern[i] = (
                pattern[i].replace("^", "").replace("$", "").replace("/", "\/")
            )

    pattern = "".join(pattern)
    pattern = "^" + pattern + "$"

    return pattern


def capitalize_first_letter(word):
    return word[:1].upper() + word[1:]



from src.core.exceptions import UnstagedFilesException, BranchAheadException
from src.core.git import has_unstaged_files, is_ahead
from src.core.models import Execution, ActionExecution
from src.core.phoenix import capitalize_first_letter
from src.core.actions.block_commit import BlockCommit
from src.core.actions.create_branch import CreateBranch
from src.core.actions.custom_actions import CustomActions
from src.core.actions.delete_branch import DeleteBranch
from src.core.actions.merge import Merge
from src.core.actions.merge import MergeRequest
from src.core.actions.rebase import Rebase
from src.core.actions.tag import Tag


def fire_rules(execution: Execution):
    if has_unstaged_files():
        raise UnstagedFilesException()

    if is_ahead():
        raise BranchAheadException()

    actions = []

    for action_execution in execution.action.get("execution"):
        do = action_execution.get("do")
        action_from_template = do.get("action")

        print(f"Validating method {action_from_template}...")

        action_class = eval(capitalize_first_letter(action_from_template))

        action = action_class(
            ActionExecution(
                execution.variables, do.get("parameters"), execution.arguments
            )
        )

        # if action.is_implemented:
        #     if not action.confirm_execution():
        #         Logger.warn(self=Checkout, msg="Execution stopped by user")
        #         sys.exit()

        actions.append(action)

    # all_actions = [
    #     action.action_execution.get("do").get("action")
    #     for action in actions.values()
    # ]

    for action in actions:
        # Logger.info(
        #     self=Rules,
        #     msg=(
        #         (
        #             "Executing method "
        #             + PythonCommons.LIGHT_GREEN
        #             + "{}"
        #             + PythonCommons.NC
        #             + "..."
        #         ).format(action.action_execution["do"]["action"])
        #     ),
        # )
        # try:
        action.execute()
        # except ExecutionException:
        #     if len(all_actions) > 0:
        #         Logger.error(
        #             self=Rules,
        #             msg=(
        #                 "An error has ocurred while processing! "
        #                 + "The following action(s) couldn't be executed:"
        #                 + PythonCommons.LIGHT_CYAN
        #                 + " {}"
        #                 + PythonCommons.NC
        #                 + "!"
        #             ).format(", ".join(all_actions)),
        #         )



from src.core.actions.executable import Executable
from src.core.logger import Logger


class Tag(Executable):

    def __init__(self, execution, action_execution):
        super().__init__(execution, action_execution)
        self.is_implemented = False

    def execute(self):
        Logger.warn(cls=Tag, msg="Not implemented yet!")

    def _parse(self):
        pass
